/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// Modifications Complete 05/14/98 Dean Polla hummer@cs.uidaho.edu

// ***************************************************************************
// Atom Class
// 
//
// Class Description:  
// ------------------
// Purpose:  This class is the main interface for the whole system.
// It instantiates and lays out the graphical user interface (GUI) components 
// found on the first interface the user will see.  It implements its own event 
// handling and allows execution of the whole system.  If this interface is 
// destroyed all subsequent interfaces called by it are destroyed.
//
// System Relationship:  This class creates the main interface for the whole 
// system.  Depending on the user event, Atom will do one of the following:
// construct the ToolInterface.class, construct the SclInterface.class, or
// execute system commands.  The system command execution is dependent on 
// audit tools assigned to a Security Classification Level (SCL).  This class
// performs file input/output.
//
// Requirements:  This class requires the Java Developemnt Kit (JDK) 1.1
// Aside from this there are no special algorithms, or constraints.  
//
// Pre-Conditions:  None.
// Post Conditions: None.
//
// Error Handling:  Addressed with the try and catch clause.
//
//
// Class Specification:
// --------------------
// This class must layout the interface components on the screen. It must
// respond to user generated events.  The following is a list of requirements
// implemented by this class:
//
//    *  Get the present working directory
//    *  Instantiate java.awt components used by the GUI
//    *  Layout the java.awt components
//    *  Provide event handling for the java.awt components
//
//
// Constructor
// -----------
//      public Atom(String title)
//
// Methods
// -------
//      public void createRegisteredAuditToolsList ()      
//      public void createSclStatusList ()
//      Main ()
//
// Event Handling 
// --------------
//      public boolean actionPerformed(ActionEvent evt)
//      public boolean itemStateChanged(ItemEvent evt)
//
// External Classes Used
// ---------------------
//      FileIO           ToolExecution
//      Box              Constrain
//
// *****************************************************************************

import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.event.*;

public class Atom extends Constrain implements ActionListener, ItemListener {

  // *******************************************
  // Interface Component Declarations
  // *******************************************
  MenuBar  menubar;                  // Menu Bar
  Menu     file, options, help;      // Menu Panes
  MenuItem quitItem, setDefaultItem, clear; // Options
  
  Panel   toolStatusPanel;           // Sub-container for Registered Audit Tools, West
  Panel   sclTypesPanel;             // Sub-container for Security Levels, East
  Panel   systemStatusPanel;         // Sub-container for System Status, South
  Panel   mainOptionsPanel;          // Sub-container for Main Options, Center

  Box     statusBox;                 // Container for audit tool status components
  Box     sclBox;                    // Container for SCL components
  Box     mainOptionsBox;            // Container for main options

  static  List    registeredToolsList;   // List for Registered Audit Tools
  static  List    sclTypesList;          // List for Security Levels

  Button  miStartButton;             // Main interface start button
  Button  miStopButton;              // Main interface stop button
  Button  miToolButton;              // Main interface tool button
  Button  miSclButton;               // Main interface SCL button

  Label   systemStatusLabel;                     // Label for the system status
  static TextArea systemStatusTextArea;          // Text area for the system status

  GridBagLayout  gridbg  = new GridBagLayout();  // Layout Manager for components
  

  // *******************************************
  // Data Declarations
  // *******************************************

  public static String sclTypesFile;      // Location of scl_types.dat file
  public static String sclTempFile;       // Temp file 
  public static String auditToolsFile;    // Location of audit_tools.dat file
  public static String auditTempFile;     // Temp file
 
  public static String sclLevelDir;       // Location of scl_Name file(s)
  public static String sclLevelTemp;      // Temp file
  public static String stopScript;        // Script to stop audit tools

  public static String directory;                       // Present working directory
  public static String toolClass = "System Security";   // Audit Tool Class Type
  
  private String[] listSelection   = new String[0];     // List selection elements
 

  // *******************************************
  // Object Instantiations 
  // *******************************************
  FileIO         atomIO   = new FileIO();
  ToolExecution  toolExec = new ToolExecution();


  // *****************************************************************************
  // public Atom(String title)
  //
  // Constructor Description:  The constructor for the Atom class.  Initializes new
  // variables and components, and lays out the components on the main interface.
  //
  // Constructor Specification:  The constructor lays out the components for 
  // each area on the main interface.  It uses the GridBagLayout Manager.  In addition
  // it defines the variables from the Data Declarations.  The following is a listing
  // of the interface layout and its corresponding components:
  // 
  //   Position  Description             Components
  //   --------  -----------             ----------
  //    * Frame  Main Interface:         GridBagLayout Manager
  //    * Menu   MenuBar:                menubar
  //    * West   Registered Audit Tools: toolStatusPanel, registeredToolsList
  //    * Center Main Screen Options:    mainOptionsPanel, miStartButton, miStopButton,
  //                                     miToolButton, miSclButton
  //    * East   Security Levels:        sclTypesPanel, sclTypesList
  //    * South  System Status:          systemStatusPanel, systemStatusTextArea
  //
  // The constructor will only be accessed once, at startup.
  // 
  // Pre-Conditions:  None.
  // Post-Conditions: None.
  //
  // *****************************************************************************

  public Atom () {
    int z =0;
    setTitle("Audit Tool Manager");    
    this.setLayout(gridbg);

    addWindowListener(new WinAdapter());

    registeredToolsList = new List(15, false);
    registeredToolsList.addItemListener(this);

    sclTypesList = new List(15, false);
    sclTypesList.addItemListener(this);

   // *** Get present working directory of Atom. ***
   String pwd = null;
   pwd = System.getProperty("user.dir");
  
   // *** Initialize Data Declarations
   directory       = pwd;
   sclTypesFile    = pwd+"/config/scl_types.dat";
   sclTempFile     = pwd+"/config/stemp.tmp";
   auditToolsFile  = pwd+"/config/audit_tools.dat";
   auditTempFile   = pwd+"/config/atemp.tmp";
   sclLevelDir     = pwd+"/levels/";
   sclLevelTemp    = pwd+"/ltemp.tmp";
   stopScript      = pwd+"/stopTool";
  

   // *** Initialize Buttons ****
   miStartButton         = new Button("START");
   miStopButton          = new Button("STOP");
   miToolButton          = new Button("TOOL");
   miSclButton           = new Button("SCL");


   // *** BUTTON LISTENERS ***
   miStartButton.addActionListener(this);
   miStopButton.addActionListener(this);
   miToolButton.addActionListener(this);
   miSclButton.addActionListener(this);


   // *** Set Font and Color of Buttons ****
   miStartButton.setFont(new Font("Dialog", Font.BOLD, 14));   
   miStopButton.setFont(new Font("Dialog", Font.BOLD, 14));
   miToolButton.setFont(new Font("Dialog", Font.BOLD, 14));
   miSclButton.setFont(new Font("Dialog", Font.BOLD, 14));

    
    // *** Create the Menu Bar **** 
     menubar = new MenuBar();
     this.setMenuBar(menubar);

        file = new Menu("File");
	file.add(quitItem = new MenuItem("Quit"));		  
	menubar.add(file);

	options = new Menu("Options");
	options.add(setDefaultItem = new MenuItem("Set Defaults"));
	options.add(clear = new MenuItem("Clear Status"));
	menubar.add(options);

	help = new Menu("Help");
	help.add(new MenuItem("About"));
	menubar.add(help);
	menubar.setHelpMenu(help);

     
    // *** MENUITEM LISTENERS ***
    quitItem.addActionListener(this);
    setDefaultItem.addActionListener(this);
    clear.addActionListener(this);
    

    // *** Registered Audit Tools ***
    createRegisteredAuditTools();

    toolStatusPanel               = new Panel();
    toolStatusPanel.setLayout(gridbg);
    constrain(toolStatusPanel, registeredToolsList, 0,0,1,1);
    statusBox = new Box(toolStatusPanel, "Registered Tools");
    statusBox.etchedOut();
   

    // *** Main Screen Options ****
    mainOptionsPanel = new Panel();

    mainOptionsPanel.setLayout(new GridLayout(4,0,0,15));
      mainOptionsPanel.add(miStartButton);
      mainOptionsPanel.add(miToolButton);
      mainOptionsPanel.add(miSclButton);
      mainOptionsPanel.add(miStopButton);
    mainOptionsBox = new Box(mainOptionsPanel, "Options");
    mainOptionsBox.etchedIn();
    mainOptionsBox.setForeground(Color.blue);

    // *** SCL Types ****
    createSclStatusList();

    sclTypesPanel                 = new Panel();
    sclTypesPanel.setLayout(gridbg);
    constrain(sclTypesPanel, sclTypesList, 0,0,1,1);
    sclBox = new Box(sclTypesPanel, "Security Levels");
    sclBox.etchedOut();

    
    // *** System Status ****
    Panel  systemStatusPanel = new Panel();
    systemStatusTextArea = new TextArea(8,50);
    systemStatusTextArea.setEditable(false);    

    systemStatusPanel.setLayout(gridbg);
    constrain(systemStatusPanel, new Label("System Status"), 0,0,1,1);
    constrain(systemStatusPanel, systemStatusTextArea, 0,1,1,1);
 

    // *** Layout components on interface ***
    constrain(this, statusBox, 0,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 15, 20, 5, 10);
    constrain(this, mainOptionsBox, 1,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.CENTER, 1.0, 1.0, 15, 10, 5, 10);
    constrain(this, sclBox, 2,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 15, 10, 5, 20);
    constrain(this, systemStatusPanel, 0,1,3,1, GridBagConstraints.HORIZONTAL,
	      GridBagConstraints.CENTER, 1.0, 1.0, 15, 10, 5, 10);

  }  // Atom Constructor



  // ***************************************************************************** 
  // public boolean actionPerformed(ActionEvent event)
  //
  // Method Description:  This method handles all of the events of type ActionEvent
  // generated by the components in the main interface.
  //
  // Method Specification:  If an ActionEvent occurs this method will handle it.
  // Events are generated by the user activating a component.  Such as, clicking a
  // button, or selecting a list item.  The following lists the events and reactions
  // for each ActionEvent:
  //
  //        Label        Component/Event    Reaction
  //        -----        ---------------    --------
  //      * SCL Start    miStartButton      Start audit tools for the selected 
  //                                        Security Level     
  //      * SCL Stop     miStopButton       Stop audit tools for the selected
  //                                        Security Level
  //      * Tool         miToolButton       Launch the ToolInterface Window
  //      * SCL          miSclButton        Launch the SclInterface Window
  //      * Menu         quitItem           Exit 
  //                     setDefaultItem     Launch DefaultWindow
  //
  // This method will be accessed many times.
  //
  // Pre-Condition:  Component objects have been instantiated.
  // Post-Condition: None.
  //
  // *****************************************************************************


  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    // *** Start SCL's ***
    if (source == miStartButton) {

      if (listSelection.length == 0) {
	systemStatusTextArea.setText("");
	systemStatusTextArea.append("Please select a Security Level and Press the Start Button. \n");
      }

      if (listSelection.length != 0) {
	  toolExec.startSclTool(listSelection[0]);
      }
    } // miStartButton


    // *** Launch the Tool Interface Window ***
    if (source == miToolButton) {
      ToolInterface tl;
      tl = new ToolInterface();
      tl.setSize(700,500);
      tl.setVisible(true);
    } // miToolButton
    

    // *** Launch the SCL Interface Window ***
    if (source == miSclButton) {
      SclInterface sl;
      sl = new SclInterface();
      sl.setSize(700,500);
      sl.setVisible(true);
    } // miSclButton

    
    // *** Stop SCL's ***
    if (source == miStopButton) {
      if (listSelection.length == 0) {
	systemStatusTextArea.setText("");
	systemStatusTextArea.append("Please select a Security Level and Press the Stop Button. \n");
      }
       
      if (listSelection.length != 0) {
	  toolExec.stopTool(listSelection[0],null);
      }
    } // miStopButton       
      
	  
    if (source == quitItem) {
      atomIO.cleanUp();
      System.exit(0);
    } // quitItem

    if (source == setDefaultItem) {
      DefaultWindow settings = new DefaultWindow(this, "Default Settings");
      settings.setVisible(true);
    } // setDefualtItem
    
    if (source == clear) {
      systemStatusTextArea.setText("");
    } // clear
  } // actionPerformed



  // *********************************************************************************
  // public boolean ItemStateChanged(ItemEvent evt)
  //
  // Method Description:  This method handles all of the events that are of type
  // ItemEvent.  These include list selection, and list deselection
  //
  // Method Specification:  If one of the preceeding events occurs this method will 
  // handle it.  Events are generated by the user activating a component.  Such as, 
  // selecting or deselecting list elements.
  //
  //        Action Type     Component/Event      Reaction
  //        -----------     ---------------      --------
  //      * LIST_SELECT     sclTypesList         listSelection = selected item
  //                        registeredToolsList  systemTextArea = "non functional"
  //      * LIST_DESELECT   sclTypesList         listSeleciton = selected item
  //
  // This method will be accessed many times.  Only the WINDOW_DESTROY will be 
  // accessed once.
  //
  // Pre-Condition:  Component objects have been instantiated.
  // Post-Condition: None.
  // 
  // *********************************************************************************

  public void itemStateChanged(ItemEvent evt) {

    // *** SCL List ***
    if (evt.getSource().equals(sclTypesList)) {
      listSelection = sclTypesList.getSelectedItems();
    }

    // *** Registered Tools List ***
    if (evt.getSource().equals(registeredToolsList)) {
	registeredToolsList.deselect(registeredToolsList.getSelectedIndex());
      systemStatusTextArea.append("The Registered Tools list is for display purposes ");
      systemStatusTextArea.append("and is non-functional.\n");
    }
     
  }  // handleEvent
  


  // *********************************************************************************
  // public void createRegisteredAuditTools ()
  //
  // Method Description:  This method instantiates the registeredToolsList and fills 
  // it with the names of all the currently registered audit tools.
  //
  // Method Specification:  First this method instantiates the registeredToolsList.  
  // This list is a no-selection list, but it defaults to a single selection list.
  // Then the list is filled through the fillList method located in the FileIO
  // class. 
  //   
  // This method will only be accessed once by the constructor.
  //
  // Pre-Condition:  registeredToolsList has been declared.
  // Post-Condition: None.
  // 
  // *********************************************************************************
  
  public void createRegisteredAuditTools () {

    registeredToolsList = atomIO.fillList(registeredToolsList, auditToolsFile, 
					   ToolInterface.NAME);
  } // createRegisteredAuditTools



  // *********************************************************************************
  // public void createSclStatusList ()
  //
  // Method Description:  This method instantiates the sclTypesList and fills 
  // it with the names of all the currently registered SCL's.
  //
  // Method Specification:  First this method instantiates the sclTypesList.  This 
  // list is a multiple selection list.  Then the list is filled through the fillList 
  // method located in the FileIO class. 
  //   
  // This method will only be accessed once by the constructor.
  //
  // Pre-Condition:  SclTypesList has been declared.
  //
  // Post-Condition: None.
  // 
  // *********************************************************************************
 
  public void createSclStatusList () {


    sclTypesList = atomIO.fillList(sclTypesList, sclTypesFile, ToolInterface.NAME);
  } // createSclStatusList
  


class WinAdapter extends WindowAdapter {
  public void windowClosing(WindowEvent event) {
    System.exit(0);
  }
}



  // *********************************************************************************
  // public static void main(String[] args) 
  //
  // Method Description:  This method executes Atom.
  //
  // Method Specification: This method initializes the Atom class, resizes the window, 
  // and shows the window.  In other words, it displays the graphical user interface
  // constructed by the Atom Class constructor.
  //
  // This method will only be accessed once.
  //
  // Pre-Condition: None.
  // Post-Condition: None. 
  // 
  // *********************************************************************************

  public static void main(String[] args) {  
    Frame f = new Atom();   
    f.pack();
    f.setVisible(true);
  } // main

} // Atom



















