/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// Modifications Complete 05/14/98 Dean Polla hummer@cs.uidaho.edu

// ***************************************************************************
// ToolInterface Class
// 
//
// Class Description:  
// ------------------
// Purpose:  This class defines the layout and functionality of the Tool Interface.
//
// This interface is a seperate window from the main interface.  This class instantiates
// and lays out the GUI components for the interface.  It implements its own 
// event handling and is capable of interacting with the whole system.  If this
// window is destroyed, only it will be destroyed.  This interface allows the
// user to add, remove, configure, start, stop, and view audit tools.
//
// System Relationship:  Intercommunication with the whole system is minimal.  The
// majority of system relationships involves updating List or Choice components 
// located on other interfaces.  For example, updating the registeredToolsList in
// the Main Interface or the Atom.class.  File input/output is performed.
//
// Requirements:  This class requires the Java Developemnt Kit (JDK) 1.1
// Aside from this there are no special algorithms, or constraints.  
//
// Pre-Conditions:  Atom.class.
// Post Conditions: True.
// Error Handling:  Addressed with the try and catch clause.
//
//
// Class Specification:
// --------------------
// This class must layout the interface components for the Tool Interface onto the 
// screen.  The Tool Interface appears on the console when the "Tool" button on the
// main interface is activated by the user.  This class will provide these options:
// add, remove, configure, execution, and view.  These options will be layed out
// in a tabular format for the user to choose from.  This class must respond to user
// generated events.  The following is a list of requirements used by this class:
//
//    *  Define audit tool parsing language (Atpl) keywords
//    *  Instantiate java.awt components used by the GUI
//    *  Layout the java.awt components
//    *  Provide event handling for the java.awt components
//
//
// Constructor
// -----------
//      public ToolInterface()
//
// Methods
// -------
//      public void layoutAddPanel()
//      public void layoutRemovePanel()
//      public void layoutConfigurePanel()
//      public void layoutExecutionPanel()
//      public void layoutViewPanel()
//      public void parseParameterText(String text)
//
// Event Handling 
// --------------
//      public boolean actionPerformed(ActionEvent evt)
//      public boolean itemStateChanged(ItemEvent evt)
//
// External Classes Used
// ---------------------
//      FileIO         ToolExecution
//      Box            Constrain 
//
// *****************************************************************************

import java.awt.*;
import java.lang.*;
import java.util.*;
import java.awt.event.*;

public class ToolInterface extends Constrain implements ActionListener, ItemListener {

  // *******************************************
  // Interface Component Declarations
  // *******************************************
  Panel   tabs;                 // Sub-container for the buttons
  Panel   cards;                // Sub-container for the button features
  Panel   addPanel;             // Sub-container for the Add feature components
  Panel   removePanel;          // Sub-container for the Remove feature components
  Panel   configurePanel;       // Sub-container for the Configure feature components
  Panel   executionPanel;       // Sub-container for the Execution feature components
  Panel   viewPanel;            // Sub-container for the View feature components

  Button  addTab;               // Add tab
  Button  removeTab;            // Remove tab
  Button  configureTab;         // Configure tab
  Button  execTab;              // Execution tab
  Button  viewTab;              // View tab
       
  Button  addSubmitButton;              // Add Submit Button
  Button  removeSubmitButton;           // Remove Submit Button
  Button  configureSubmitButton;        // Configure Submit Button
  Button  executionSubmitButton;        // Start Submit Button
  Button  continueButton;               // Continue button 

  TextField  addNameTextField;           // Add Feature: Name text field
  TextField  addParameterTextField;      // Add Feature: Parameter text field
  TextField  startParametersTextField;   // Start Feature: Parameter text field
  TextField  startDirectoryTextField;    // Start Feature: Directory text field

  TextArea   addDescriptionTextArea;     // Text area for the Add description
  TextArea   configureTextArea;          // Text area for the Configure current configuration
  TextArea   configureTextAreaEdit;      // Text area for the Configure new configuration
  static TextArea   viewTextArea;        // Text area for the View feature
  
  List  removeToolList;              // Tool List for the Remove feature
  List  configureToolList;           // Tool List for the Configure feature
  List  viewToolList;                // Tool List for the View feature
  List  startToolList;               // Tool List for the Start feature
  
  static Choice  addChoice;          // Choice menu for the Add feature
         Choice  configureChoice;    // Choice menu for the Configure feature
  static Choice  startChoice;        // Choice menu for the Start feature
  
  Checkbox       startAssignCheckBox;      // Checkbox for the Assignment feature
  Checkbox       startCheckBox;            // Assignemt start tool
  Checkbox       stopCheckBox;             // Assignment stop tool
  Checkbox       addDaemonCheckbox;        // Add daemon checkbox
  Checkbox       addProcessCheckbox;       // Add process checkbox
  Checkbox       startProcessTypeCheckBox; // Execution process type
  Checkbox       execStopType;             // Execution stop type

  CheckboxGroup  addExecType;              // CheckboxGroup for Add
  CheckboxGroup  executionGroup;           // CheckboxGroup for Execution
  
  CardLayout singleCard   = new CardLayout();      // Layout Manager for the features / cards
  GridBagLayout  gridbag  = new GridBagLayout();   // Layout Manager for the components
  
  public static String auditToolName;              // Variable used by AddWindow Class

  
  // *******************************************
  // Data Declarations
  // *******************************************

  static  String NAME        = "NAME";              // Atpl keyword 
  static  String PROCESS     = "PROCESS";           // Atpl keyword
          String SCLTYPE     = "SCLTYPE";           // Atpl keyword  
          String PARAMETER   = "PARAMETER";         // Atpl keyword  
  static  String DESCRIPTION = "DESCRIPTION";       // Atpl keyword  
  static  String TOOLCLASS   = "TOOLCLASS";         // Atpl keyword  
  static  String DIRECTORY   = "DIRECTORY";         // Atpl keyword  
  static  String DEFAULT     = "DEFAULT";           // Atpl keyword  
  static  String END         = "END";               // Atpl keyword  
  static  String ENDSYMBOL   = "#@";                // Atpl symbol

  static  int    proceed       = 0;                 // Flag set by AddWindow.class

  static String  process = "process";               // Keyword to define process type
         String  daemon  = "daemon";                // Keyword to define process type
 
  private String[] alphaListSelection = new String[0];   // List selection elements (alpha names)
  private String selectedItem;

  // *******************************************
  // Object instantiation
  // *******************************************
  
  FileIO        toolIO   = new FileIO();
  ToolExecution toolExec = new ToolExecution();

 
  // *****************************************************************************
  // public ToolInterface()
  //
  // Constructor Description:  The constructor for the ToolInterface  class.  Initializes
  // new variables and components, and lays out the components on the tool interface.
  //
  // Constructor Specification:  The constructor lays out the components for 
  // each area on the tool interface.  It uses the GridBagLayout Manager.  In addition
  // it defines the variables from the Data Declarations.  The following is a listing
  // of the interface layout and its corresponding components:
  // 
  //   Position  Description             Components
  //   --------  -----------             ----------
  //    * Frame  Tool Interface:         GridBagLayout Manager
  //    * North  Tabs/Features Panel     GridLayout Manager, addTab, removeTab, 
  //                                     configureButton, execButton, stopButton, and
  //                                     viewButton
  //    * Center Cards                   CardLayout Manager
  //             Add                     addPanel, addChoice, addNameTextField, 
  //                                     addParameterTextField, addDescriptionTextArea 
  //             Remove                  removePanel, removeToolList, panel1
  //             Configure               configurePanel, configureToolList, configureChoice,
  //                                     configureTextArea, configureTextAreaEdit,panel1,
  //                                     panel2
  //             Execution               executionPanel, startExecTextField, 
  //                                     startDirectoryTextField, startToolList, startChoice,
  //                                     startCheckBox
  //             View                    viewPanel, viewToolList, viewTextArea                
  //
  // The constructor will be accessed each time ToolInterface , is instantiated.
  // 
  // Pre-Conditions:  True.
  // Post-Conditions: True.
  //
  // *****************************************************************************

  public ToolInterface () {
    setTitle("Registered Tool Options");

    addWindowListener(new WinAdapter());
    
    // *** Initialize Buttons ****
    addTab           = new Button("Add");
    removeTab        = new Button("Remove");
    configureTab     = new Button("Configure");
    execTab          = new Button("Execution");
    viewTab          = new Button("View");
    continueButton   = new Button("continue");

    // *** BUTTON LISTENERS ***
    addTab.addActionListener(this);
    removeTab.addActionListener(this);
    configureTab.addActionListener(this);
    execTab.addActionListener(this);
    viewTab.addActionListener(this);
    continueButton.addActionListener(this);

    // *** Set Font and Color of Buttons ****
    addTab.setFont(new Font("Dialog", Font.BOLD, 14));
    removeTab.setFont(new Font("Dialog", Font.BOLD, 14));
    configureTab.setFont(new Font("Dialog", Font.BOLD, 14));
    execTab.setFont(new Font("Dialog", Font.BOLD, 14));
    viewTab.setFont(new Font("Dialog", Font.BOLD, 14));
    continueButton.setFont(new Font("Dialog", Font.BOLD, 12));
    continueButton.setForeground(Color.blue);

    // *** Tabs ****
    tabs = new Panel();
    tabs.setLayout(new GridLayout(0,6));
     tabs.add(addTab);
     tabs.add(removeTab);   
     tabs.add(configureTab);
     tabs.add(execTab);
     tabs.add(viewTab);
    add("North", tabs);

    // *** Layout Card Method ****
    layoutAddPanel();

    // *** Card ****
    cards = new Panel();
    cards.setLayout(singleCard);
    cards.add("Add",       addPanel);
    add("Center", cards);
  }  // Constructor



  // ***************************************************************************** 
    // public boolean actionPerformed(ActionEvent evt)
  //
  // Method Description: This method handles all of the events of type ActionEvent
  // generated by the components in the tool interface.
  //
  // Method Specification:  If an ActionEvent occurs this method will handle it.
  // Events are generated by the user activating a component.  Such as, clicking a
  // button, or selecting a list item.  The following lists the events and reactions
  // for each ActionEvent:
  //
  //        Label     Component/Event       Reaction
  //        -----     ---------------       --------
  //      * Add       continueButton        First interface of adding an audit tool, when
  //                                        this button is clicked the second interface
  //                                        appears as a dialog box     
  //      * Remove    removeSubmitButton    Remove an audit tool from the registered audit
  //                                        tools
  //      * Configure configureSubmitButton Modify the user selected field of the audtit
  //                                        tool
  //      * Execution executionSubmitButton Start the user selected audit tool or
  //                                        Stop the user selected audit tool
  //
  // This method will be accessed many times.
  //
  // Pre-Condition:  Component objects have been instantiated.
  // Post-Condition: True. Events handle properly.
  //
  // *****************************************************************************

  public void actionPerformed(ActionEvent evt) {
    Object source = evt.getSource();
    
    String descriptionInfo = "";
    String fieldData       = "";
    int    tempIndex;


    // -----------------------------------------------------------------------
    // ADD
    //
    //   * Text Field: <name> 
    //   * Text Field: <parameters> 
    //   * Text Area:  <description>
    //   * Choice Menu
    // -----------------------------------------------------------------------

    if (source == continueButton) {
	String nameInfo = addNameTextField.getText();
	String sclInfo  = addChoice.getSelectedItem();
	String parmInfo = addParameterTextField.getText();
	descriptionInfo = addDescriptionTextArea.getText();        
	nameInfo = nameInfo.trim();                                

      // VALIDITY check
      if (nameInfo.equals("")) {
	Atom.systemStatusTextArea.setText("The NAME field is required to Add a tool. \n");
	return;
      }
      
      // Variable used by AddWindow Class
      auditToolName = nameInfo;
      
      // Initialize and show the AddWindow Dialog.
      AddWindow popup = new AddWindow(this, "Add Tool");
      popup.setVisible(true);

      //
      // If the user clicked the Submit button in the AddWindow Dialog.
      // Then do the following:
      //        * Put the Audit Tool Data Atom.auditToolsFile
      //        * Data is in the format <name> <scl type> <tool class> 
      //          <directory> <default> <description> <end symbol>
      //        * Update the registered audit tools list components
      //        * Clear the TextFields of the Add interface

      if (proceed == 1) {	  
	toolIO.AtplAppendText(Atom.auditToolsFile, NAME, nameInfo);
	toolIO.AtplAppendText(Atom.auditToolsFile, SCLTYPE, sclInfo);
      
	if (addProcessCheckbox.getState()) {
	    toolIO.AtplAppendText(Atom.auditToolsFile, PROCESS, process);
	}
	else {
	    toolIO.AtplAppendText(Atom.auditToolsFile, PROCESS, daemon);
	}

	parseParameterText(parmInfo);
	  
	toolIO.AtplAppendText(Atom.auditToolsFile, TOOLCLASS, AddWindow.toolClassInfo);
	toolIO.AtplAppendText(Atom.auditToolsFile, DIRECTORY, AddWindow.directoryInfo);
	toolIO.AtplAppendText(Atom.auditToolsFile, DEFAULT,   AddWindow.defaultInfo);     
	
	toolIO.AtplAppendText(Atom.auditToolsFile, DESCRIPTION, descriptionInfo);
	toolIO.AtplAppendText(Atom.auditToolsFile, END, ENDSYMBOL);
	
	// UPDATE registered audit tools
	Atom.registeredToolsList.add(nameInfo);

	// The SclInterface class may not have been instantiated yet, so 
	// check and handle this list if possible.
	try { 	       
	  SclInterface.assignToolList.add(nameInfo);
	}
	catch (NullPointerException e) {
	  String err = e.toString();
	}

	// Clear the text fields
	addNameTextField.setText("");
	addParameterTextField.setText("");
	addDescriptionTextArea.setText("");	
      }
    } // add continueButton


    // -----------------------------------------------------------------------
    // REMOVE
    //
    //   * Registered Tools List 
    // -----------------------------------------------------------------------
    if (source == removeSubmitButton) {
      
	// VALIDITY check
	if (selectedItem == null) {
	    Atom.systemStatusTextArea.setText("Please select an Item to Remove. \n");
	    return;
	}
      else {
	  // UPDATE ALL Registed Tools Lists
	  removeToolList.remove(selectedItem);
	  Atom.registeredToolsList.remove(selectedItem);

	  // The Scl class may not have been instantiated yet, so 
	  // check and handle these lists if possible.
	  try { 	       
	      SclInterface.assignToolList.remove(selectedItem); 	       
	  }
	  catch (NullPointerException e) {
	      String err = e.toString();
	  }
	
	  // UPDATE Atom.auditToolsFile
	  toolIO.AtplRemoveRecord(Atom.auditToolsFile, Atom.auditTempFile, selectedItem);	   
	  selectedItem=null;
      }
    } // remove button


    // -----------------------------------------------------------------------
    // CONFIGURE
    //
    //   * Choice Menu: <name> <scltype> <parameter> <tool class> <directory>
    //                   <default> <description>
    //   * Text Area: <new information>
    //   * Registered Tools List
    // -----------------------------------------------------------------------
    if (source == configureSubmitButton) {
	String choiceItem = configureChoice.getSelectedItem();
	String info       = configureTextAreaEdit.getText();
	info = info.trim();
      
	// VALIDITY check
	if (!(alphaListSelection.length > 0)) {
	    Atom.systemStatusTextArea.append("Please select an item from the list to configure.\n");
	    return;
	}

	// VALIDITY check
	if (info.equals("")) {
	    Atom.systemStatusTextArea.append("Please enter New Configuration information. \n");
	    return;
	}

	toolIO.AtplReplaceRecord(Atom.auditToolsFile, Atom.auditTempFile, alphaListSelection[0], 
				 choiceItem, info);

	// UPDATE all Registered Tool Lists
	if (choiceItem.equals(NAME)) {
	    int index = configureToolList.getSelectedIndex();
	    Atom.registeredToolsList.replaceItem(info,index);
	    configureToolList.replaceItem(info,index);
	    alphaListSelection[0] = info;
	}

	// DISPLAY
	configureTextArea.setText("");
	configureTextAreaEdit.setText("");
	toolIO.AtplGetRecord(Atom.auditToolsFile, alphaListSelection[0], configureTextArea);  
    } // Configure Button


    // -----------------------------------------------------------------------
    // EXECUTION
    //
    //    * Choice Menu: <name> <scltype> <parameter> <tool class> <directory>
    //                   <default> <description>
    //   * Text Area: <new information>
    //   * Registered Tools List
    // -----------------------------------------------------------------------
    if (source == executionSubmitButton) {
	String execCommand    = startParametersTextField.getText();
	String directoryInfo  = startDirectoryTextField.getText();
	
	// VALIDITY check --> Text Fields
	if ((execCommand.equals("")) || (directoryInfo.equals(""))) {
	    Atom.systemStatusTextArea.append("Please enter information in Directory and Execution Command Text Fields. \n");
	    return;
	}

	// VALIDITY check --> List
	if (!(alphaListSelection.length > 0)) {
	    Atom.systemStatusTextArea.append("Please select an item from the Registered Tools list.\n");
	    return;
	}

	if (!(directoryInfo.endsWith("/"))) {            // if directory does not end with "/" then put it there
	    directoryInfo = directoryInfo+"/";
	}	 
	
	if (executionGroup.getSelectedCheckbox().equals(stopCheckBox)) {
	    if (execStopType.getState()) {
		toolExec.stopTool(null,execCommand);
	    }
	    else {
		toolExec.stopTool(null,alphaListSelection[0]);
	    }
	}
	else {
	    if (startAssignCheckBox.getState()) {
		String sclInfo     = toolIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], SCLTYPE);
		String processInfo = toolIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], PROCESS);
		toolIO.appendSclAssignment(sclInfo, processInfo, execCommand, directoryInfo);  		
	    }
	    if (startProcessTypeCheckBox.getState()) {
		toolExec.startDaemon(directoryInfo, execCommand);
	    }
	    else {
		toolExec.startProcess(directoryInfo, execCommand);		
	    }
	}
    } // Start



    // ----------------------
    // Cards
    // ----------------------
    if (source instanceof Component &&
	((Component)evt.getSource()).getParent().equals(tabs)) {

      String tabType = evt.getActionCommand();

 
      //  
      // If the user selects a tab/feature display the appropriate card, moreover
      // display the card/features interface
      //
      if (tabType.equals("Add")) {
	layoutAddPanel();
	cards.add("Add", addPanel); 
      }      

      if (tabType.equals("Remove")) {
	layoutRemovePanel();
	selectedItem=null;
	cards.add("Remove", removePanel); 
      }	

      if (tabType.equals("Configure")) {
	layoutConfigurePanel();
	cards.add("Configure", configurePanel); 
	alphaListSelection = new String[0];
      }	

      if (tabType.equals("Execution")) {
	layoutExecutionPanel();
	cards.add("Execution", executionPanel); 
	alphaListSelection = new String[0];
      }	
	
      if (tabType.equals("View")) {
	layoutViewPanel();
	cards.add("View", viewPanel); 
	alphaListSelection = new String[0];
      }	

      singleCard.show(cards, tabType);   
    }

  } // action


class WinAdapter extends WindowAdapter {
  public void windowClosing(WindowEvent event) {
    dispose();
  }
}


  // *********************************************************************************
  // public boolean ItemStateChanged(ItemEvent evt)
  //
  // Method Description:  This method handles all of the events that are of type
  // ItemEvent.  These include list selection, and list deselection
  //
  // Method Specification:  If one of the preceeding events occurs this method will 
  // handle it.  Events are generated by the user activating a component.  Such as, 
  // selecting or deselecting list elements.
  //
  //        Action Type     Component/Event        Reaction
  //        -----------     ---------------        --------
  //      * LIST_SELECT     removeToolList         alphalistSelection = selected item
  //                        configureToolsList     alphaListSelection = selected item
  //                        execToolList           alphaListSelection = selected item
  //                        viewToolList           alphaListSelection = selected item
  //      * LIST_DESELECT   removeToolLst          alphalistSelection = selected item
  //                        configureToolsList     alphaListSelection = selected item
  //                        execToolList           alphaListSelection = selected item
  //                        viewToolList           alphaListSelection = selected item
  //
  // This method will be accessed many times.  Only the WINDOW_DESTROY will be 
  // accessed once.
  //
  // Pre-Condition:  Component objects have been instantiated.
  // Post-Condition: True.  Lists have proper component names or indexes.
  // 
  // *********************************************************************************

  public void itemStateChanged(ItemEvent event) {
    String fieldData = "";
    
    switch(event.getStateChange()) {
    case ItemEvent.SELECTED:

      // Remove
      if (event.getSource().equals(removeToolList)) {
	selectedItem = removeToolList.getSelectedItem();
      }    

      // Configure
      //     * Get List selection
      //     * Clear TextField
      //     * Place record information in TextField based on List selection
      //
      if (event.getSource().equals(configureToolList)) {
      	alphaListSelection = configureToolList.getSelectedItems();
	configureTextArea.setText("");
	toolIO.AtplGetRecord(Atom.auditToolsFile, alphaListSelection[0], configureTextArea);      
      }

      // Start
      //     * Get List selection
      //     * Clear TextFields
      //     * Get field information from audit tool records based on List selection
      //     * Place information in appropriate TextField
      //
      if (event.getSource().equals(startToolList)) {
	alphaListSelection = startToolList.getSelectedItems();
	startParametersTextField.setText("");
	startDirectoryTextField.setText("");
	fieldData = toolIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], DEFAULT);
	startParametersTextField.setText(fieldData);
	fieldData = toolIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], DIRECTORY);
	startDirectoryTextField.setText(fieldData);
      }

      // View
      //    * Get List selection
      //    * Clear TextField
      //    * Place information in TextField
      //
      if (event.getSource().equals(viewToolList)) {
	alphaListSelection = viewToolList.getSelectedItems();
	viewTextArea.setText("");
	toolIO.AtplGetRecord(Atom.auditToolsFile, alphaListSelection[0], viewTextArea);
      }
      
    break;

    case ItemEvent.DESELECTED:
      
      // Configure
      if (event.getSource().equals(configureToolList)) {
	alphaListSelection = configureToolList.getSelectedItems();		
      }

      // Start
      if (event.getSource().equals(startToolList)) {
	alphaListSelection = startToolList.getSelectedItems();	
      }
      
      // View
      if (event.getSource().equals(viewToolList)) {
	alphaListSelection = viewToolList.getSelectedItems();	
      }
    break;
    } // switch 
    
  }



  // *********************************************************************************
  // public void layoutAddPanel ()
  //
  // Method Description:  This method lays out all of the components for the Add tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Add tab is activated by the user, the corresponding
  // Add feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Add tab is activated this
  // method will be called.
  //
  // Pre-Condition:  Add components have been declared.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutAddPanel () {
    addPanel                 = new Panel();
    addChoice                = new Choice();
    addNameTextField         = new TextField(30);
    addParameterTextField    = new TextField(50);
    addDescriptionTextArea   = new TextArea(11,60);
    addExecType              = new CheckboxGroup();
    addDaemonCheckbox        = new Checkbox("Daemon (Wait)", addExecType, true);
    addProcessCheckbox       = new Checkbox("Process (No Wait)", addExecType, false);

    Panel insetPanelA        = new Panel();
    Panel insetPanelB        = new Panel();
    Panel insetPanelC        = new Panel();

    insetPanelA.setLayout(new GridLayout(2,2,60,5));
    insetPanelB.setLayout(new GridLayout(2,1));
    insetPanelC.setLayout(new GridLayout(2,1));

    addPanel = new Panel();    
    addPanel.setLayout(gridbag);
    
    addChoice = toolIO.fillChoice(addChoice, Atom.sclTypesFile, NAME);
    
    addDaemonCheckbox.setFont(new Font("Courier", Font.PLAIN, 12));
    addProcessCheckbox.setFont(new Font("Courier", Font.PLAIN, 12));
    addChoice.setFont(new Font("Courier", Font.PLAIN, 12));

    insetPanelA.add(new Label("Default Security Level"));
    insetPanelA.add(addDaemonCheckbox);
    insetPanelA.add(addChoice);
    insetPanelA.add(addProcessCheckbox);
    insetPanelB.add(new Label("Parameters"));
    insetPanelB.add(addParameterTextField);
    insetPanelC.add(insetPanelA);
    insetPanelC.add(insetPanelB);

    Box PanelABox = new Box(insetPanelC, "Options");
    PanelABox.etchedIn();
    
    
    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.
    constrain(addPanel, new Label("Name:"), 0,0,1,1, 0,20,0,0);
    constrain(addPanel, addNameTextField, 0,1,1,1, 0,20,0,0);     

    constrain(addPanel, PanelABox, 0,2,1,1, 0,20,0,0);

    constrain(addPanel, new Label("Description:"), 0,3,1,1, 10,20,0,0);
    constrain(addPanel, addDescriptionTextArea, 0,4,1,1, 0,20,0,0);   
    constrain(addPanel, continueButton, 0,5,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);

  }  // layoutAddPanel


  // *********************************************************************************
  // public void layoutRemovePanel ()
  //
  // Method Description:  This method lays out all of the components for the Remove tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Remove tab is activated by the user, the corresponding
  // Remove feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Remove tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Remove components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutRemovePanel () {
    removePanel         = new Panel();             
    removeToolList      = new List(20, false);       
    Panel panel1        = new Panel();
   
    panel1.setLayout(gridbag);
    removePanel.setLayout(gridbag);
    removeSubmitButton = new Button("Submit");
    removeSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
    removeSubmitButton.setForeground(Color.blue);

    // ADD LISTENERS
    removeSubmitButton.addActionListener(this);
    removeToolList.addItemListener(this);

    // Fill the removeToolList with the names of the registered audit tools
    removeToolList = toolIO.fillList(removeToolList, Atom.auditToolsFile, NAME);

    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.          
    constrain(panel1, removeToolList, 0,0,1,1);
    Box panelBox = new Box(panel1, "Registered Tools");
    panelBox.etchedOut();
     
    constrain(removePanel, panelBox, 0,0,1,1);
    constrain(removePanel, removeSubmitButton, 0,1,1,1, 20,40,0,0);

  } // layoutRemovePanel()


  // *********************************************************************************
  // public void layoutConfigurePanel ()
  //
  // Method Description:  This method lays out all of the components for the Configure tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Configure tab is activated by the user, the corresponding
  // Configure feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Configure tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Configure components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutConfigurePanel() {
    configurePanel              = new Panel();
    configureToolList           = new List(22, false);
    configureChoice             = new Choice();
    configureTextArea           = new TextArea(8,50);
    configureTextArea.setEditable(false);
    configureTextAreaEdit       = new TextArea(8,50);

    Panel     panel1            = new Panel();
    Panel     panel2            = new Panel();
    
    configurePanel.setLayout(gridbag);
    panel1.setLayout(gridbag);
    panel2.setLayout(gridbag);
    configureSubmitButton = new Button("Submit");
    configureSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
    configureSubmitButton.setForeground(Color.blue);

    // ADD LISTENERS
    configureSubmitButton.addActionListener(this);
    configureToolList.addItemListener(this);
    
    // Fill the configureToolList with the names of the registered audit tools     
    configureToolList = toolIO.fillList(configureToolList, Atom.auditToolsFile, NAME);
     
    // This is where the choice menu is filled with choices
    configureChoice.add(NAME);
    configureChoice.add(PROCESS);
    configureChoice.add(SCLTYPE);
    configureChoice.add(PARAMETER);
    configureChoice.add(TOOLCLASS);
    configureChoice.add(DIRECTORY);
    configureChoice.add(DEFAULT);
    configureChoice.add(DESCRIPTION);
    configureChoice.setFont(new Font("Courier", Font.PLAIN, 12));
        
    constrain(panel1, configureToolList, 0,0,1,1);
    Box panelBox = new Box(panel1, "Registered Tools");
    panelBox.etchedOut();

    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.               
    constrain(panel2, configureChoice, 0,0,1,1, 10,0,0,0);
    constrain(panel2, new Label("Current Configuration"), 0,1,1,1, 20,0,0,0);
    constrain(panel2, configureTextArea, 0,2,1,1);
    constrain(panel2, new Label("New Configuration"), 0,3,1,1, 10,0,0,0);
    constrain(panel2, configureTextAreaEdit, 0,4,1,1); 
    constrain(panel2, configureSubmitButton, 0,5,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);

    constrain(configurePanel, panelBox, 0,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 20, 10, 5, 10);
    constrain(configurePanel, panel2, 1,0,1,1, GridBagConstraints.NONE, 
	       GridBagConstraints.NORTH, 1.0, 1.0, 20, 0, 5, 10);    	

  } // layoutRemovePanel()


  // *********************************************************************************
  // public void layoutExecutionPanel ()
  //
  // Method Description:  This method lays out all of the components for the Start tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Start tab is activated by the user, the corresponding
  // Start feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Start tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Start components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutExecutionPanel() {
    executionPanel                      = new Panel();
    startParametersTextField        = new TextField(60);
    startDirectoryTextField         = new TextField(60);
    startToolList                   = new List(15, false);
    startChoice                     = new Choice();
    executionGroup                  = new CheckboxGroup();
    startCheckBox                   = new Checkbox("Start Registered Tool", executionGroup, true);
    stopCheckBox                    = new Checkbox("Stop Registered Tool    ", executionGroup, false);
    startAssignCheckBox                   = new Checkbox("Assign Tool to Security Level");
    startProcessTypeCheckBox        = new Checkbox("Daemon?");
    execStopType                    = new Checkbox("Use Execution Command?");


    Panel insetPanelA               = new Panel();
    Panel insetPanelB               = new Panel();
    Panel insetPanelC               = new Panel();
    Panel insetPanelD               = new Panel();
    Panel panel1                    = new Panel();
    Panel panel2                    = new Panel();
   
    insetPanelA.setLayout(new FlowLayout());
    insetPanelB.setLayout(gridbag);
    insetPanelC.setLayout(gridbag);
    insetPanelD.setLayout(gridbag);

    executionPanel.setLayout(gridbag);
    panel1.setLayout(gridbag);
    panel2.setLayout(gridbag);

    executionSubmitButton = new Button("Submit");
    executionSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
    executionSubmitButton.setForeground(Color.blue);

    startCheckBox.setFont(new Font("Courier", Font.PLAIN, 12));
    stopCheckBox.setFont(new Font("Courier", Font.PLAIN, 12));
    
    // ADD LISTENERS
    executionSubmitButton.addActionListener(this);
    startToolList.addItemListener(this);
    
    // Fill the startToolList with the names of the registered audit tools
    // Fill the choice menu with the names of the SCL types
    startToolList = toolIO.fillList(startToolList, Atom.auditToolsFile, NAME);
    startChoice   = toolIO.fillChoice(startChoice, Atom.sclTypesFile, NAME);

    // Layout Registered Tools Lists
    insetPanelA.add(startToolList);
    Box panelABox = new Box(insetPanelA, "Registered Tools");
    panelABox.etchedOut();
    
    constrain(insetPanelB, startCheckBox, 0,0,1,1, 0,0,0,0);
    constrain(insetPanelB, startAssignCheckBox, 0,1,1,1, 10,0,0,0);
    constrain(insetPanelB, startProcessTypeCheckBox, 0,2,1,1, 10,0,0,0);

    Box panelBBox = new Box(insetPanelB, "Start Registered Tool");
    panelBBox.etchedIn();

    constrain(insetPanelC, stopCheckBox, 0,0,1,1, 0,0,0,0);
    constrain(insetPanelC, execStopType, 0,1,1,1, 10,0,0,0);
    execStopType.setState(true);
    Box panelCBox = new Box(insetPanelC, "Stop Registered Tool");
    panelCBox.etchedIn();

    constrain(insetPanelD, panelBBox, 0,0,1,1, 10,0,0,0);
    constrain(insetPanelD, panelCBox, 0,1,1,1, 30,0,0,0);

    constrain(panel1, panelABox, 0,0,1,1, 10,0,0,30);
    constrain(panel1, insetPanelD,  1,0,1,1, 10,30,0,0);

    constrain(panel2, new Label("Directory:"), 0,0,1,1);
    constrain(panel2, startDirectoryTextField, 0,1,1,1);
    constrain(panel2, new Label("Execution Command:"), 0,2,1,1);
    constrain(panel2, startParametersTextField, 0,3,1,1);
    constrain(panel2, executionSubmitButton, 0,4,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);
    
    constrain(executionPanel, panel1, 0,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 10, 0, 0, 0);
    constrain(executionPanel, panel2, 0,0,1,1, GridBagConstraints.HORIZONTAL,
	      GridBagConstraints.SOUTH, 1.0, 1.0, 10, 20, 0, 0);

  } // layoutExecutionPanel()



  // *********************************************************************************
  // public void layoutViewPanel ()
  //
  // Method Description:  This method lays out all of the components for the View tab,
  // located on the tool interface.
  //
  // Method Specification:  When the View tab is activated by the user, the corresponding
  // View feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the View tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  View components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutViewPanel() {
    viewPanel                    = new Panel();
    viewToolList                 = new List(20, false);
    viewTextArea                 = new TextArea(23,50);
    viewTextArea.setEditable(false);

    Panel     panel1             = new Panel();
    Panel     panel2             = new Panel();

    viewPanel.setLayout(gridbag);
    panel1.setLayout(gridbag);
    panel2.setLayout(gridbag);

    // ADD LISTENERS
    viewToolList.addItemListener(this);
    
    // Fill the viewToolList with the names of the registered audit tools   
    viewToolList = toolIO.fillList(viewToolList, Atom.auditToolsFile, NAME);     

    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.                        
    constrain(panel1, viewToolList, 0,0,1,1);
    Box panelBox = new Box(panel1, "Audit Tools");
    panelBox.etchedOut();
    
    constrain(panel2, new Label("Audit Tool Specifications"), 0,0,1,1);
    constrain(panel2, viewTextArea, 0,1,1,1);    
    
    constrain(viewPanel, panelBox, 0,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 20, 0, 0, 0);
    constrain(viewPanel, panel2, 1,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 20, 10, 0, 10);

  } // layoutViewPanel()
    

  // *********************************************************************************
  // public void parseParameterText (String text)
  //
  // Method Description:  This method parses the parameter string passed to it into 
  // seperate tokens/parameters.  
  //
  // Method Specification:  This method parses the parameter string passed to it.  The
  // parsing is delimited by a space (" ").  Each token is considered a parameter for 
  // the audit tool, and it is placed into the audit_tools.dat file in the format:
  // PARAMETER  token.
  //
  // This method will be accessed several times.  Every time the Add tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  True.  The string passed to this method does not have to be in
  // any special format.
  //
  // Post-Condition: True.  All of the individual (parsed) parameters are placed into
  // the audit_tools.dat file.
  // 
  // *********************************************************************************

  public void parseParameterText(String text) {
    String token;
    StringTokenizer inputString = new StringTokenizer(text, " ");
    
    while (inputString.hasMoreTokens()) {
      token = inputString.nextToken();
      toolIO.AtplAppendText(Atom.auditToolsFile, PARAMETER, token);
    } // while

  } // parseParameterText

}

 











 




